<?php

namespace Drupal\view_cache_invalidate_on_update\EventSubscriber;

use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Invalidates View cache when a View is updated.
 */
class ViewCacheInvalidationSubscriber implements EventSubscriberInterface {

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * Constructs a new ViewCacheInvalidationSubscriber.
   *
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache_tags_invalidator
   *   The cache tags invalidator.
   */
  public function __construct(CacheTagsInvalidatorInterface $cache_tags_invalidator) {
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ConfigEvents::SAVE => ['onConfigSave', 100],
    ];
  }

  /**
   * Invalidates View cache when a View config is updated.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The configuration save event.
   */
  public function onConfigSave(ConfigCrudEvent $event) {
    $config = $event->getConfig();

    // Only act on updates to existing configuration (not new creations)
    if ($config->isNew()) {
      return;
    }

    $config_name = $config->getName();

    // Check if the updated config is a View
    if (str_starts_with($config_name, 'views.view.')) {
      // Extract the View ID from the config name
      $view_id = substr($config_name, strlen('views.view.'));

      // Create cache tag
      $cache_tag = 'config:views.view.' . $view_id;

      // Invalidate the cache tag
      $this->cacheTagsInvalidator->invalidateTags([$cache_tag]);
    }
  }

}
